@@ -19,138 +19,98 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-/**
- * SECTION:completioncontext
- * @title: GtkSourceCompletionContext
- * @short_description: The context of a completion
- *
- * Initially, the completion window is hidden. For a completion to occur, it has
- * to be activated. The different possible activations are listed in
- * #GtkSourceCompletionActivation. When an activation occurs, a
- * #GtkSourceCompletionContext object is created, and the eligible providers are
- * asked to add proposals with gtk_source_completion_context_add_proposals().
- *
- * If no proposals are added, the completion window remains hidden, and the
- * context is destroyed.
- *
- * On the other hand, if proposals are added, the completion window becomes
- * visible, and the user can choose a proposal. If the user is not happy with
- * the shown proposals, he or she can insert or delete characters, to modify the
- * completion context and therefore hoping to see the proposal he or she wants.
- * This means that when an insertion or deletion occurs in the #GtkTextBuffer
- * when the completion window is visible, the eligible providers are again asked
- * to add proposals. The #GtkSourceCompletionContext:activation remains the
- * same in this case.
- *
- * When the completion window is hidden, the interactive completion is triggered
- * only on insertion in the buffer, not on deletion. Once the completion window
- * is visible, then on each insertion or deletion, there is a new population and
- * the providers are asked to add proposals. If there are no more proposals, the
- * completion window disappears. So if you want to keep the completion window
- * visible, but there are no proposals, you can insert a dummy proposal named
- * "No proposals". For example, the user types progressively the name of
- * a function, and some proposals appear. The user types a bad character and
- * there are no proposals anymore. What the user wants is to delete the last
- * character, and see the previous proposals. If the completion window
- * disappears, the previous proposals will not reappear on the character
- * deletion.
- *
- * A #GtkTextIter is associated with the context, this is where the completion
- * takes place. With this #GtkTextIter, you can get the associated
- * #GtkTextBuffer with gtk_text_iter_get_buffer().
- */
-
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #include "gtksourcecompletioncontext.h"
-#include "gtksourceview-enumtypes.h"
+#include "gtksourceview-typebuiltins.h"
 #include "gtksourcecompletionprovider.h"
 #include "gtksourceview-i18n.h"
 #include "gtksourcecompletion.h"
 
+#define GTK_SOURCE_COMPLETION_CONTEXT_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE((object), GTK_SOURCE_TYPE_COMPLETION_CONTEXT, GtkSourceCompletionContextPrivate))
+
 struct _GtkSourceCompletionContextPrivate
 {
 	GtkSourceCompletion *completion;
 
 	GtkTextMark *mark;
 	GtkSourceCompletionActivation activation;
+
+	gulong mark_set_id;
 };
 
+/* Properties */
 enum
 {
 	PROP_0,
+
 	PROP_COMPLETION,
 	PROP_ITER,
 	PROP_ACTIVATION
 };
 
+/* Signals */
 enum
 {
 	CANCELLED,
-	N_SIGNALS
+	NUM_SIGNALS
 };
 
-static guint context_signals[N_SIGNALS];
+guint context_signals[NUM_SIGNALS] = {0,};
 
-G_DEFINE_TYPE_WITH_PRIVATE (GtkSourceCompletionContext, gtk_source_completion_context, G_TYPE_INITIALLY_UNOWNED)
+G_DEFINE_TYPE (GtkSourceCompletionContext, gtk_source_completion_context, G_TYPE_INITIALLY_UNOWNED)
+
+/* FIXME: we use this util to get the buffer from the completion
+   but this object is not robust to a change of the buffer associated
+   to the view. Context lifetime should be short enough to not really
+   matter.
+*/
+
+static GtkTextBuffer *
+get_buffer (GtkSourceCompletionContext *context)
+{
+	GtkSourceView *view = gtk_source_completion_get_view (context->priv->completion);
+	return gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
+}
 
 static void
 gtk_source_completion_context_dispose (GObject *object)
 {
-	GtkSourceCompletionContext *context = GTK_SOURCE_COMPLETION_CONTEXT (object);
+	GtkSourceCompletionContext *context;
+	GtkTextBuffer *buffer;
 
-	if (context->priv->mark != NULL)
+	context = GTK_SOURCE_COMPLETION_CONTEXT (object);
+	buffer = get_buffer (context);
+
+	if (context->priv->mark_set_id)
 	{
-		GtkTextBuffer *buffer = gtk_text_mark_get_buffer (context->priv->mark);
-
-		if (buffer != NULL)
-		{
-			gtk_text_buffer_delete_mark (buffer, context->priv->mark);
-		}
-
-		g_object_unref (context->priv->mark);
-		context->priv->mark = NULL;
+		g_signal_handler_disconnect (buffer, context->priv->mark_set_id);
+		context->priv->mark_set_id = 0;
 	}
 
 	g_clear_object (&context->priv->completion);
 
+	if (context->priv->mark)
+	{
+		gtk_text_buffer_delete_mark (buffer, context->priv->mark);
+		context->priv->mark = NULL;
+	}
+
 	G_OBJECT_CLASS (gtk_source_completion_context_parent_class)->dispose (object);
 }
 
 static void
-set_iter (GtkSourceCompletionContext *context,
-	  GtkTextIter                *iter)
+gtk_source_completion_context_set_iter (GtkSourceCompletionContext *context,
+                                        GtkTextIter                *iter)
 {
-	GtkTextBuffer *buffer;
-
-	buffer = gtk_text_iter_get_buffer (iter);
-
-	if (context->priv->mark != NULL)
-	{
-		GtkTextBuffer *old_buffer;
-
-		old_buffer = gtk_text_mark_get_buffer (context->priv->mark);
-
-		if (old_buffer != buffer)
-		{
-			g_object_unref (context->priv->mark);
-			context->priv->mark = NULL;
-		}
-	}
+	GtkTextBuffer *buffer = get_buffer (context);
 
 	if (context->priv->mark == NULL)
 	{
 		context->priv->mark = gtk_text_buffer_create_mark (buffer, NULL, iter, FALSE);
-		g_object_ref (context->priv->mark);
 	}
 	else
 	{
 		gtk_text_buffer_move_mark (buffer, context->priv->mark, iter);
 	}
-
-	g_object_notify (G_OBJECT (context), "iter");
 }
 
 static void
@@ -166,15 +126,12 @@ gtk_source_completion_context_set_property (GObject      *object,
 		case PROP_COMPLETION:
 			context->priv->completion = g_value_dup_object (value);
 			break;
-
 		case PROP_ITER:
-			set_iter (context, g_value_get_boxed (value));
+			gtk_source_completion_context_set_iter (context, (GtkTextIter *) g_value_get_boxed (value));
 			break;
-
 		case PROP_ACTIVATION:
 			context->priv->activation = g_value_get_flags (value);
 			break;
-
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 	}
@@ -193,32 +150,56 @@ gtk_source_completion_context_get_property (GObject    *object,
 		case PROP_COMPLETION:
 			g_value_set_object (value, context->priv->completion);
 			break;
-
 		case PROP_ITER:
 			{
 				GtkTextIter iter;
-
-				if (gtk_source_completion_context_get_iter (context, &iter))
-				{
-					g_value_set_boxed (value, &iter);
-				}
+				gtk_source_completion_context_get_iter (context, &iter);
+				g_value_set_boxed (value, &iter);
 			}
 			break;
-
 		case PROP_ACTIVATION:
 			g_value_set_flags (value, context->priv->activation);
 			break;
-
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 	}
 }
 
+static void
+buffer_mark_set_cb (GtkTextBuffer              *buffer,
+                    GtkTextIter                *iter,
+                    GtkTextMark                *mark,
+                    GtkSourceCompletionContext *context)
+{
+	if (mark == context->priv->mark)
+	{
+		g_object_notify (G_OBJECT (context), "iter");
+	}
+}
+
+static void
+gtk_source_completion_context_constructed (GObject *object)
+{
+	GtkSourceCompletionContext *context;
+	GtkTextBuffer *buffer;
+
+	/* we need to connect after the completion property is set */
+	context = GTK_SOURCE_COMPLETION_CONTEXT (object);
+	buffer = get_buffer (context);
+	context->priv->mark_set_id = g_signal_connect (buffer,
+	                                               "mark-set",
+	                                               G_CALLBACK (buffer_mark_set_cb),
+	                                               context);
+
+	G_OBJECT_CLASS (gtk_source_completion_context_parent_class)->constructed (object);
+}
+
 static void
 gtk_source_completion_context_class_init (GtkSourceCompletionContextClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
+	object_class->constructed = gtk_source_completion_context_constructed;
 	object_class->set_property = gtk_source_completion_context_set_property;
 	object_class->get_property = gtk_source_completion_context_get_property;
 	object_class->dispose = gtk_source_completion_context_dispose;
@@ -235,8 +216,11 @@ gtk_source_completion_context_class_init (GtkSourceCompletionContextClass *klass
 		              G_TYPE_FROM_CLASS (klass),
 		              G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
 		              G_STRUCT_OFFSET (GtkSourceCompletionContextClass, cancelled),
-		              NULL, NULL, NULL,
-		              G_TYPE_NONE, 0);
+		              NULL,
+		              NULL,
+		              g_cclosure_marshal_VOID__VOID,
+		              G_TYPE_NONE,
+		              0);
 
 	/**
 	 * GtkSourceCompletionContext:completion:
@@ -246,12 +230,10 @@ gtk_source_completion_context_class_init (GtkSourceCompletionContextClass *klass
 	g_object_class_install_property (object_class,
 	                                 PROP_COMPLETION,
 	                                 g_param_spec_object ("completion",
-	                                                      "Completion",
-	                                                      "The completion object to which the context belongs",
+	                                                      _("Completion"),
+	                                                      _("The completion object to which the context belongs"),
 	                                                      GTK_SOURCE_TYPE_COMPLETION,
-	                                                      G_PARAM_READWRITE |
-							      G_PARAM_CONSTRUCT_ONLY |
-							      G_PARAM_STATIC_STRINGS));
+	                                                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
 	/**
 	 * GtkSourceCompletionContext:iter:
@@ -260,12 +242,12 @@ gtk_source_completion_context_class_init (GtkSourceCompletionContextClass *klass
 	 **/
 	g_object_class_install_property (object_class,
 	                                 PROP_ITER,
-					 g_param_spec_boxed ("iter",
-							     "Iterator",
-							     "The GtkTextIter at which the completion was invoked",
-							     GTK_TYPE_TEXT_ITER,
-							     G_PARAM_READWRITE |
-							     G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_boxed ("iter",
+	/* Translators: The GtkTextIter at which the completion was invoked */
+	                                                      _("Iterator"),
+	                                                      _("The GtkTextIter at which the completion was invoked"),
+	                                                      GTK_TYPE_TEXT_ITER,
+	                                                      G_PARAM_READWRITE));
 
 	/**
 	 * GtkSourceCompletionContext:activation:
@@ -275,34 +257,32 @@ gtk_source_completion_context_class_init (GtkSourceCompletionContextClass *klass
 	g_object_class_install_property (object_class,
 	                                 PROP_ACTIVATION,
 	                                 g_param_spec_flags ("activation",
-	                                                     "Activation",
-	                                                     "The type of activation",
+	                                                     _("Activation"),
+	                                                     _("The type of activation"),
 	                                                     GTK_SOURCE_TYPE_COMPLETION_ACTIVATION,
-	                                                     GTK_SOURCE_COMPLETION_ACTIVATION_USER_REQUESTED,
-	                                                     G_PARAM_READWRITE |
-							     G_PARAM_CONSTRUCT |
-							     G_PARAM_STATIC_STRINGS));
+	                                                     GTK_SOURCE_COMPLETION_ACTIVATION_NONE,
+	                                                     G_PARAM_READWRITE));
+
+	g_type_class_add_private (object_class, sizeof(GtkSourceCompletionContextPrivate));
 }
 
 static void
 gtk_source_completion_context_init (GtkSourceCompletionContext *context)
 {
-	context->priv = gtk_source_completion_context_get_instance_private (context);
+	context->priv = GTK_SOURCE_COMPLETION_CONTEXT_GET_PRIVATE (context);
 }
 
 /**
  * gtk_source_completion_context_add_proposals:
  * @context: a #GtkSourceCompletionContext.
- * @provider: a #GtkSourceCompletionProvider.
- * @proposals: (nullable) (element-type GtkSource.CompletionProposal): The list of proposals to add.
+ * @provider: (type GtkSource.CompletionProvider): a #GtkSourceCompletionProvider.
+ * @proposals: (element-type GtkSource.CompletionProposal): The list of proposals to add.
  * @finished: Whether the provider is finished adding proposals.
  *
  * Providers can use this function to add proposals to the completion. They
  * can do so asynchronously by means of the @finished argument. Providers must
  * ensure that they always call this function with @finished set to %TRUE
  * once each population (even if no proposals need to be added).
- * Population occurs when the gtk_source_completion_provider_populate()
- * function is called.
  **/
 void
 gtk_source_completion_context_add_proposals (GtkSourceCompletionContext  *context,
@@ -327,49 +307,27 @@ gtk_source_completion_context_add_proposals (GtkSourceCompletionContext  *contex
  *
  * Get the iter at which the completion was invoked. Providers can use this
  * to determine how and if to match proposals.
- *
- * Returns: %TRUE if @iter is correctly set, %FALSE otherwise.
  **/
-gboolean
+void
 gtk_source_completion_context_get_iter (GtkSourceCompletionContext *context,
                                         GtkTextIter                *iter)
 {
-	GtkTextBuffer *mark_buffer;
-	GtkSourceView *view;
-	GtkTextBuffer *completion_buffer;
+	GtkTextBuffer *buffer;
 
-	g_return_val_if_fail (GTK_SOURCE_IS_COMPLETION_CONTEXT (context), FALSE);
+	g_return_if_fail (GTK_SOURCE_IS_COMPLETION_CONTEXT (context));
 
-	if (context->priv->mark == NULL)
+	buffer = get_buffer (context);
+
+	if (context->priv->mark != NULL)
+	{
+		gtk_text_buffer_get_iter_at_mark (buffer, iter, context->priv->mark);
+	}
+	else
 	{
 		/* This should never happen: context should be always be created
 		   providing a position iter */
 		g_warning ("Completion context without mark");
-		return FALSE;
 	}
-
-	mark_buffer = gtk_text_mark_get_buffer (context->priv->mark);
-
-	if (mark_buffer == NULL)
-	{
-		return FALSE;
-	}
-
-	view = gtk_source_completion_get_view (context->priv->completion);
-	if (view == NULL)
-	{
-		return FALSE;
-	}
-
-	completion_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
-
-	if (completion_buffer != mark_buffer)
-	{
-		return FALSE;
-	}
-
-	gtk_text_buffer_get_iter_at_mark (mark_buffer, iter, context->priv->mark);
-	return TRUE;
 }
 
 /**
@@ -383,8 +341,7 @@ gtk_source_completion_context_get_iter (GtkSourceCompletionContext *context,
 GtkSourceCompletionActivation
 gtk_source_completion_context_get_activation (GtkSourceCompletionContext *context)
 {
-	g_return_val_if_fail (GTK_SOURCE_IS_COMPLETION_CONTEXT (context),
-			      GTK_SOURCE_COMPLETION_ACTIVATION_NONE);
+	g_return_val_if_fail (GTK_SOURCE_IS_COMPLETION_CONTEXT (context), FALSE);
 
 	return context->priv->activation;
 }
@@ -398,8 +355,7 @@ _gtk_source_completion_context_cancel (GtkSourceCompletionContext *context)
 }
 
 GtkSourceCompletionContext *
-_gtk_source_completion_context_new (GtkSourceCompletion *completion,
-				    GtkTextIter         *position)
+_gtk_source_completion_context_new (GtkSourceCompletion *completion, GtkTextIter *position)
 {
 	g_return_val_if_fail (GTK_SOURCE_IS_COMPLETION (completion), NULL);
 	g_return_val_if_fail (position != NULL, NULL);
