@@ -19,18 +19,13 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #include "gtksourcemark.h"
 #include "gtksourcebuffer.h"
-#include "gtksourcebuffer-private.h"
 #include "gtksourceview-i18n.h"
 
 /**
  * SECTION:mark
- * @Short_description: Mark object for GtkSourceBuffer
+ * @Short_description: mark object for #GtkSourceBuffer
  * @Title: GtkSourceMark
  * @See_also: #GtkSourceBuffer
  *
@@ -39,12 +34,13 @@
  * the text has changed though its position may change.
  *
  * #GtkSourceMark<!-- -->s are organised in categories which you have to set
- * when you create the mark. Each category can have a priority, a pixbuf and
- * other associated attributes. See gtk_source_view_set_mark_attributes().
- * The pixbuf will be displayed in the margin at the line where the mark
- * residents if the #GtkSourceView:show-line-marks property is set to %TRUE. If
- * there are multiple marks in the same line, the pixbufs will be drawn on top
- * of each other. The mark with the highest priority will be drawn on top.
+ * when you create the mark. Each category can have a pixbuf and a priority
+ * associated using #gtk_source_view_set_mark_category_pixbuf and
+ * #gtk_source_view_set_mark_category_priority. The pixbuf will be displayed in
+ * the margin at the line where the mark residents if the
+ * #GtkSourceView::show-line-marks property is set to TRUE. If there are
+ * multiple marks in the same line, the pixbufs will be drawn on top of each
+ * other. The mark with the highest priority will be drawn on top.
  */
 
 enum
@@ -58,7 +54,7 @@ struct _GtkSourceMarkPrivate
 	gchar *category;
 };
 
-G_DEFINE_TYPE_WITH_PRIVATE (GtkSourceMark, gtk_source_mark, GTK_TYPE_TEXT_MARK);
+G_DEFINE_TYPE (GtkSourceMark, gtk_source_mark, GTK_TYPE_TEXT_MARK);
 
 static void
 gtk_source_mark_set_property (GObject      *object,
@@ -141,18 +137,19 @@ gtk_source_mark_class_init (GtkSourceMarkClass *klass)
 	g_object_class_install_property (object_class,
 					 PROP_CATEGORY,
 					 g_param_spec_string ("category",
-							      "Category",
-							      "The mark category",
+							      _("category"),
+							      _("The mark category"),
 							      NULL,
-							      G_PARAM_READWRITE |
-							      G_PARAM_CONSTRUCT_ONLY |
-							      G_PARAM_STATIC_STRINGS));
+							      G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
+
+	g_type_class_add_private (object_class, sizeof (GtkSourceMarkPrivate));
 }
 
 static void
 gtk_source_mark_init (GtkSourceMark *mark)
 {
-	mark->priv = gtk_source_mark_get_instance_private (mark);
+	mark->priv = G_TYPE_INSTANCE_GET_PRIVATE (mark, GTK_SOURCE_TYPE_MARK,
+						  GtkSourceMarkPrivate);
 }
 
 /**
@@ -167,7 +164,7 @@ gtk_source_mark_init (GtkSourceMark *mark)
  * If name is NULL, the mark is anonymous; otherwise, the mark can be retrieved
  * by name using gtk_text_buffer_get_mark().
  * Normally marks are created using the utility function
- * gtk_source_buffer_create_source_mark().
+ * gtk_source_buffer_create_mark().
  *
  * Returns: a new #GtkSourceMark that can be added using gtk_text_buffer_add_mark().
  *
@@ -207,14 +204,14 @@ gtk_source_mark_get_category (GtkSourceMark *mark)
 /**
  * gtk_source_mark_next:
  * @mark: a #GtkSourceMark.
- * @category: (nullable): a string specifying the mark category, or %NULL.
+ * @category: (allow-none): a string specifying the mark category, or %NULL.
  *
  * Returns the next #GtkSourceMark in the buffer or %NULL if the mark
  * was not added to a buffer. If there is no next mark, %NULL will be returned.
  *
  * If @category is %NULL, looks for marks of any category.
  *
- * Returns: (nullable) (transfer none): the next #GtkSourceMark, or %NULL.
+ * Returns: (transfer none): the next #GtkSourceMark, or %NULL.
  *
  * Since: 2.2
  */
@@ -227,15 +224,11 @@ gtk_source_mark_next (GtkSourceMark *mark,
 	g_return_val_if_fail (GTK_SOURCE_IS_MARK (mark), NULL);
 
 	buffer = gtk_text_mark_get_buffer (GTK_TEXT_MARK (mark));
-
-	if (buffer == NULL)
-	{
+	if (buffer != NULL)
+		return _gtk_source_buffer_source_mark_next (GTK_SOURCE_BUFFER (buffer),
+							    mark, category);
+	else
 		return NULL;
-	}
-
-	return _gtk_source_buffer_source_mark_next (GTK_SOURCE_BUFFER (buffer),
-						    mark,
-						    category);
 }
 
 /**
@@ -248,7 +241,7 @@ gtk_source_mark_next (GtkSourceMark *mark,
  *
  * If @category is %NULL, looks for marks of any category
  *
- * Returns: (nullable) (transfer none): the previous #GtkSourceMark, or %NULL.
+ * Returns: (transfer none): the previous #GtkSourceMark, or %NULL.
  *
  * Since: 2.2
  */
@@ -261,14 +254,10 @@ gtk_source_mark_prev (GtkSourceMark *mark,
 	g_return_val_if_fail (GTK_SOURCE_IS_MARK (mark), NULL);
 
 	buffer = gtk_text_mark_get_buffer (GTK_TEXT_MARK (mark));
-
-	if (buffer == NULL)
-	{
+	if (buffer != NULL)
+		return _gtk_source_buffer_source_mark_prev (GTK_SOURCE_BUFFER (buffer),
+							    mark, category);
+	else
 		return NULL;
-	}
-
-	return _gtk_source_buffer_source_mark_prev (GTK_SOURCE_BUFFER (buffer),
-						    mark,
-						    category);
 }
 
